from typing import TypedDict, Literal


class Coordinates(TypedDict):
    lat: float
    lng: float


class StructuredAddress(TypedDict, total=False):
    locale: str
    country_name: str
    country_code: str
    state: str
    city: str
    borough: str
    neighborhood: str
    street: str
    number: str
    complements: str
    postal_code: str


class TransactionAddress(TypedDict, total=False):
    type: Literal['shipping', 'billing', 'home']
    structured_address: StructuredAddress
    address_coordinates: Coordinates


class PaymentValue(TypedDict):
    amount: float
    currency: str


class CardInfo(TypedDict, total=False):
    bin: str
    last_four_digits: str
    expiry_year: str
    expiry_month: str


class PaymentMethod(TypedDict, total=False):
    type: Literal['credit', 'debit']
    credit_card_info: CardInfo
    debit_card_info: CardInfo



from typing import Final


class FeedbackEvents:
    SIGNUP_ACCEPTED: Final[str] = 'signup_accepted'
    SIGNUP_DECLINED: Final[str] = 'signup_declined'
    PAYMENT_ACCEPTED: Final[str] = 'payment_accepted'
    PAYMENT_ACCEPTED_BY_THIRD_PARTY: Final[str] = 'payment_accepted_by_third_party'
    PAYMENT_ACCEPTED_BY_CONTROL_GROUP: Final[str] = 'payment_accepted_by_control_group'
    PAYMENT_DECLINED: Final[str] = 'payment_declined'
    PAYMENT_DECLINED_BY_RISK_ANALYSIS: Final[str] = \
        'payment_declined_by_risk_analysis'
    PAYMENT_DECLINED_BY_MANUAL_REVIEW: Final[str] = \
        'payment_declined_by_manual_review'
    PAYMENT_DECLINED_BY_BUSINESS: Final[str] = 'payment_declined_by_business'
    PAYMENT_DECLINED_BY_ACQUIRER: Final[str] = 'payment_declined_by_acquirer'
    LOGIN_ACCEPTED: Final[str] = 'login_accepted'
    LOGIN_DECLINED: Final[str] = 'login_declined'
    VERIFIED: Final[str] = 'verified'
    IDENTITY_FRAUD: Final[str] = 'identity_fraud'
    ACCOUNT_TAKEOVER: Final[str] = 'account_takeover'
    CHARGEBACK_NOTIFICATION: Final[str] = 'chargeback_notification'
    CHARGEBACK: Final[str] = 'chargeback'
    MPOS_FRAUD: Final[str] = 'mpos_fraud'
    CHALLENGE_PASSED: Final[str] = 'challenge_passed'
    CHALLENGE_FAILED: Final[str] = 'challenge_failed'
    PASSWORD_CHANGED_SUCCESSFULLY: Final[str] = 'password_changed_successfully'
    PASSWORD_CHANGE_FAILED: Final[str] = 'password_change_failed'
    PROMOTION_ABUSE: Final[str] = 'promotion_abuse'



from typing import Final


class Endpoints:
    BASE: Final[str] = 'https://api.incognia.com'

    TOKEN: Final[str] = f'{BASE}/api/v2/token'
    SIGNUPS: Final[str] = f'{BASE}/api/v2/onboarding/signups'
    FEEDBACKS: Final[str] = f'{BASE}/api/v2/feedbacks'
    TRANSACTIONS: Final[str] = f'{BASE}/api/v2/authentication/transactions'



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='incognia-python',
    version='1.2.0',
    packages=find_packages(exclude=['tests']),
    license='MIT',
    url='https://github.com/inloco/incognia-python',
    author='Incognia',
    author_email='incognia@incognia.com',
    description='Incognia API Python Client',
    python_requires='>=3.8',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ]
)



import datetime as dt
import json
from typing import Final, Optional, List

import requests

from .datetime_util import total_milliseconds_since_epoch
from .endpoints import Endpoints
from .exceptions import IncogniaHTTPError, IncogniaError
from .json_util import encode
from .models import Coordinates, StructuredAddress, TransactionAddress, PaymentValue, PaymentMethod
from .token_manager import TokenManager


class IncogniaAPI:
    JSON_CONTENT_HEADER: Final[dict] = {'Content-type': 'application/json'}

    def __init__(self, client_id: str, client_secret: str):
        self.__token_manager = TokenManager(client_id, client_secret)

    def __get_authorization_header(self) -> dict:
        access_token, token_type = self.__token_manager.get()
        return {'Authorization': f'{token_type} {access_token}'}

    def register_new_signup(self,
                            installation_id: str,
                            address_line: Optional[str] = None,
                            structured_address: Optional[StructuredAddress] = None,
                            address_coordinates: Optional[Coordinates] = None) -> dict:
        if not installation_id:
            raise IncogniaError('installation_id is required.')

        try:
            headers = self.__get_authorization_header()
            headers.update(self.JSON_CONTENT_HEADER)
            body = {
                'installation_id': installation_id,
                'address_line': address_line,
                'structured_address': structured_address,
                'address_coordinates': address_coordinates
            }
            data = encode(body)
            response = requests.post(Endpoints.SIGNUPS, headers=headers, data=data)
            response.raise_for_status()

            return json.loads(response.content.decode('utf-8'))

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None

    def get_signup_assessment(self, signup_id: str) -> dict:
        if not signup_id:
            raise IncogniaError('signup_id is required.')

        try:
            headers = self.__get_authorization_header()
            response = requests.get(f'{Endpoints.SIGNUPS}/{signup_id}', headers=headers)
            response.raise_for_status()

            return json.loads(response.content.decode('utf-8'))

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None

    def register_feedback(self,
                          event: str,
                          timestamp: dt.datetime,
                          external_id: Optional[str] = None,
                          login_id: Optional[str] = None,
                          payment_id: Optional[str] = None,
                          signup_id: Optional[str] = None,
                          account_id: Optional[str] = None,
                          installation_id: Optional[str] = None) -> None:
        if not event:
            raise IncogniaError('event is required.')
        if not timestamp:
            raise IncogniaError('timestamp is required.')

        try:
            headers = self.__get_authorization_header()
            headers.update(self.JSON_CONTENT_HEADER)
            body = {
                'event': event,
                'timestamp': total_milliseconds_since_epoch(timestamp),
                'external_id': external_id,
                'login_id': login_id,
                'payment_id': payment_id,
                'signup_id': signup_id,
                'account_id': account_id,
                'installation_id': installation_id
            }
            data = encode(body)
            response = requests.post(Endpoints.FEEDBACKS, headers=headers, data=data)
            response.raise_for_status()

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None

    def register_payment(self,
                         installation_id: str,
                         account_id: str,
                         external_id: Optional[str] = None,
                         addresses: Optional[List[TransactionAddress]] = None,
                         payment_value: Optional[PaymentValue] = None,
                         payment_methods: Optional[List[PaymentMethod]] = None,
                         evaluate: Optional[bool] = None) -> dict:
        if not installation_id:
            raise IncogniaError('installation_id is required.')
        if not account_id:
            raise IncogniaError('account_id is required.')

        try:
            headers = self.__get_authorization_header()
            headers.update(self.JSON_CONTENT_HEADER)
            params = None if evaluate is None else {'eval': evaluate}
            body = {
                'type': 'payment',
                'installation_id': installation_id,
                'account_id': account_id,
                'external_id': external_id,
                'addresses': addresses,
                'payment_value': payment_value,
                'payment_methods': payment_methods
            }
            data = encode(body)
            response = requests.post(Endpoints.TRANSACTIONS, headers=headers, params=params,
                                     data=data)
            response.raise_for_status()

            return json.loads(response.content.decode('utf-8'))

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None

    def register_login(self,
                       installation_id: str,
                       account_id: str,
                       external_id: Optional[str] = None,
                       evaluate: Optional[bool] = None) -> dict:
        if not installation_id:
            raise IncogniaError('installation_id is required.')
        if not account_id:
            raise IncogniaError('account_id is required.')

        try:
            headers = self.__get_authorization_header()
            headers.update(self.JSON_CONTENT_HEADER)
            params = None if evaluate is None else {'eval': evaluate}
            body = {
                'type': 'login',
                'installation_id': installation_id,
                'account_id': account_id,
                'external_id': external_id
            }
            data = encode(body)
            response = requests.post(Endpoints.TRANSACTIONS, headers=headers, params=params,
                                     data=data)
            response.raise_for_status()

            return json.loads(response.content.decode('utf-8'))

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None



import base64
import datetime as dt
import json
from typing import Final, Optional, NamedTuple

import requests

from .endpoints import Endpoints
from .exceptions import IncogniaHTTPError


class TokenValues(NamedTuple):
    access_token: str
    token_type: str


class TokenManager:
    TOKEN_REFRESH_BEFORE_SECONDS: Final[int] = 10

    def __init__(self, client_id: str, client_secret: str):
        self.__client_id: str = client_id
        self.__client_secret: str = client_secret
        self.__token_values: Optional[TokenValues] = None
        self.__expiration_time: Optional[dt.datetime] = None

    def __refresh_token(self) -> None:
        client_id, client_secret = self.__client_id, self.__client_secret
        client_id_and_secret_encoded = base64.urlsafe_b64encode(
            f'{client_id}:{client_secret}'.encode('ascii')).decode('utf-8')
        headers = {'Authorization': f'Basic {client_id_and_secret_encoded}'}

        try:
            response = requests.post(url=Endpoints.TOKEN, headers=headers,
                                     auth=(client_id, client_secret))
            response.raise_for_status()

            parsed_response = json.loads(response.content.decode('utf-8'))
            token_values = TokenValues(parsed_response['access_token'],
                                       parsed_response['token_type'])
            expiration_time = dt.datetime.now() + dt.timedelta(
                seconds=int(parsed_response['expires_in']))

            self.__token_values, self.__expiration_time = token_values, expiration_time

        except requests.HTTPError as e:
            raise IncogniaHTTPError(e) from None

    def __is_expired(self) -> bool:
        return (self.__expiration_time - dt.datetime.now()).total_seconds() <= \
               self.TOKEN_REFRESH_BEFORE_SECONDS

    def get(self) -> TokenValues:
        if not self.__expiration_time or self.__is_expired():
            self.__refresh_token()
        return self.__token_values
